import{s as a,c as s,a as e,C as t}from"./chunk-B4BG7PRW-Bi7i6fBg.js";import{_ as i}from"./index-p2UKDQnp.js";import"./chunk-FMBD7UC4-CD3DMf4O.js";import"./chunk-55IACEB6-BBOUhanf.js";import"./chunk-QN33PNHL-DcMriKYW.js";var p={parser:e,get db(){return new t},renderer:s,styles:a,init:i(r=>{r.class||(r.class={}),r.class.arrowMarkerAbsolute=r.arrowMarkerAbsolute},"init")};export{p as diagram};
